/*!***********************************************************************
 *************************************************************************
 * \file            lcd_driver.c
 *
 * \author          copyright 1999-2004
 *                  Kai Klenovsek
 *                  http://kk.elektronik-4u.de
 *
 * \date            2004-07-27
 *
 * \b LICENSE: <br>
 *       This code is distributed under the GNU Public License <br>
 *       which can be found at http://www.gnu.org/licenses/gpl.txt <br>
 *
***************************************************************************
***************************************************************************/

/* 2008: modified by Hans - Joachim Borchers
 *       - custom chars adapted
 *       - LCD init. sequency changed
 */

/**************************************************************************
*
* INCLUDES
*
***************************************************************************/

#include <avr/io.h>
#include <avr/pgmspace.h>
#include "typedef.h"
#include "sysdef.h"
#include "hwdef.h"
#include "drvlcd.h"



/**************************************************************************
*
* MODUL GLOBAL VARS
*
***************************************************************************/

#if LCD_TYP == 0
static unsigned char actcntr = 1;                    /*!< Actual active lcd controller */
#endif

// custom LCD characters
unsigned char __attribute__ ((progmem)) LcdCustomChar[] =
{
  0x00, 0x1F, 0x00, 0x00, 0x00, 0x00, 0x1F, 0x00, // 0. 0/5 full progress block
  0x00, 0x1F, 0x10, 0x10, 0x10, 0x10, 0x1F, 0x00, // 1. 1/5 full progress block
  0x00, 0x1F, 0x18, 0x18, 0x18, 0x18, 0x1F, 0x00, // 2. 2/5 full progress block
  0x00, 0x1F, 0x1C, 0x1C, 0x1C, 0x1C, 0x1F, 0x00, // 3. 3/5 full progress block
  0x00, 0x1F, 0x1E, 0x1E, 0x1E, 0x1E, 0x1F, 0x00, // 4. 4/5 full progress block
  0x00, 0x1F, 0x1F, 0x1F, 0x1F, 0x1F, 0x1F, 0x00, // 5. 5/5 full progress block
  0x03, 0x07, 0x0F, 0x1F, 0x0F, 0x07, 0x03, 0x00, // 6. rewind arrow
  0x18, 0x18, 0x03, 0x04, 0x04, 0x04, 0x03, 0x00  // 7. C 
//  0x00, 0x1F, 0x1F, 0x1F, 0x1F, 0x1F, 0x1F, 0x00, // 7. stop block
//  0x1B, 0x1B, 0x1B, 0x1B, 0x1B, 0x1B, 0x1B, 0x00, // 8. pause bars
//  0x18, 0x1C, 0x1E, 0x1F, 0x1E, 0x1C, 0x18, 0x00, // 9. fast-forward arrow
//  0x00, 0x04, 0x04, 0x0E, 0x0E, 0x1F, 0x1F, 0x00, // 10. scroll up arrow
//  0x00, 0x1F, 0x1F, 0x0E, 0x0E, 0x04, 0x04, 0x00, // 11. scroll down arrow
//  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // 12. blank character
//  0x00, 0x0E, 0x19, 0x15, 0x13, 0x0E, 0x00, 0x00, // 13. animated play icon frame 0
//  0x00, 0x0E, 0x15, 0x15, 0x15, 0x0E, 0x00, 0x00, // 14. animated play icon frame 1
//  0x00, 0x0E, 0x13, 0x15, 0x19, 0x0E, 0x00, 0x00, // 15. animated play icon frame 2
//  0x00, 0x0E, 0x11, 0x1F, 0x11, 0x0E, 0x00, 0x00, // 16. animated play icon frame 3
};



/**************************************************************************
*
* INTERNAL PROTOTYPES
*
**************************************************************************/

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
/*!\fn static void lcd_waitbusy( void );
 * \brief Wait until lcd is ready */
static void lcd_waitbusy( void );



//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
/*!\fn static void lcd_trig_enable( void );
 * \brief Controll enable pin from lcd */
static void lcd_trig_enable( void );



/**************************************************************************
*
* DEFINES
*
***************************************************************************/

// Lcd DDRAM register for 4x40 display
#if LCD_TYP == 0
#define MAXLINELENGHT 40
prog_char screenpos[2] = { 0x80, 0xC0 };
#endif


// Lcd DDRAM register for 2x40 display
#if LCD_TYP == 1
#define MAXLINELENGHT 40
prog_char screenpos[2] = { 0x80, 0xC0 };
#endif


// Lcd DDRAM register for 2x24 display
#if LCD_TYP == 2
#define MAXLINELENGHT 24
prog_char screenpos[2] = { 0x80, 0xC0 };
#endif


// Lcd DDRAM register for 4x20 display
#if LCD_TYP == 3
#define MAXLINELENGHT 20
prog_char screenpos[4] = { 0x80, 0xC0, 0x94, 0xD4 };
#endif


// Lcd DDRAM register for 4x16 display
#if LCD_TYP == 4
#define MAXLINELENGHT 16
prog_char screenpos[4] = { 0x80, 0xC0, 0x90, 0xD0 };
#endif



//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

extern void lcd_init( void )

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
{
    register unsigned char i = 1;
    unsigned char  n; 
#if LCD_MODE == 4
    LCD_DDR_PORT = LCD_DDR_PORT | 0xF0;          // Set data port to ouput
#else
    LCD_DDR_PORT = 0xFF;                         // Set data port to ouput
#endif    
    RW_DDR |= (1<<RW);                           // Set r/w port to output
    RS_DDR |= (1<<RS);                           // Set cmd or data port to output
    ENA_DDR |= (1<<ENA);                         // Set enable port to output

    // Only if 4x40 lcd with second enable pin
    // is selected
#if LCD_TYP == 0
    ENA2_DDR |= (1<<ENA2);                       // Set enable 2 port to output
#endif

    // 8 bit mode part
#if LCD_MODE == 8
    //delay_ms(15);
    for(n=0;n<60;n++) delay_us(1000);

#if LCD_TYP == 0
    // Init two controller
    for (i=1; i<=2; i++)
    {
#endif
        lcd_sendcom(LCD_INIT, i);                 // Diverse init managment
        //delay_ms(5);
        for(n=0;n<5;n++) delay_us(1000);
        lcd_sendcom(LCD_INIT, i);
        delay_us(150);
        lcd_sendcom(LCD_INIT, i);
        lcd_sendcom(LCD_8_2LINE, i);
        lcd_sendcom(LCD_ON, i);
        lcd_sendcom(LCD_CLEAR, i);
        lcd_sendcom(LCD_ENTRY_6, i);

        
#if LCD_TYP == 0
        actcntr = 2;
    }
    actcntr = 1;
#endif
#endif

    // 4 bit mode part
#if LCD_MODE == 4
    //delay_ms(15);                             // Diverse init managment
    for(n=0;n<15;n++) delay_us(1000);

#if LCD_TYP == 0
    // Init two controller
    for (i=1; i<=2; i++)
    {
#endif
        LCD_DATA_PORT = (LCD_DATA_PORT & 0x0F) | LCD_INIT;
        lcd_trig_enable();
        //delay_ms(5);
        for(n=0;n<5;n++) delay_us(1000);
        LCD_DATA_PORT = (LCD_DATA_PORT & 0x0F) | LCD_INIT;
        lcd_trig_enable();
        delay_us(150);
        LCD_DATA_PORT = (LCD_DATA_PORT & 0x0F) | LCD_INIT;
        lcd_trig_enable();
        LCD_DATA_PORT = (LCD_DATA_PORT & 0x0F) | LCD_4_1LINE;
        lcd_trig_enable();
        lcd_sendcom(LCD_4_2LINE, i);
        lcd_sendcom(LCD_ON, i);
        lcd_sendcom(LCD_CLEAR, i);
        lcd_sendcom(LCD_ENTRY_6, i);
#if LCD_TYP == 0
        actcntr = 2;
    }
    actcntr = 1;
#endif
#endif

#if LCD_TYP == 0
    // Init two controller
    for (i=1; i<=2; i++)
    {
#endif
      // load the first 8 custom characters
      lcd_LoadCustomChar((u08*)LcdCustomChar,0,0,i);
      lcd_LoadCustomChar((u08*)LcdCustomChar,1,1,i);
      lcd_LoadCustomChar((u08*)LcdCustomChar,2,2,i);
      lcd_LoadCustomChar((u08*)LcdCustomChar,3,3,i);
      lcd_LoadCustomChar((u08*)LcdCustomChar,4,4,i);
      lcd_LoadCustomChar((u08*)LcdCustomChar,5,5,i);
      lcd_LoadCustomChar((u08*)LcdCustomChar,6,6,i);
      lcd_LoadCustomChar((u08*)LcdCustomChar,7,7,i);
#if LCD_TYP == 0
        actcntr = 2;
    }
    actcntr = 1;
#endif


}



//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

static void lcd_waitbusy( void )

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
{
    LCD_DDR_PORT &= ~(1<<BUSY);
    RS_LOW();
    RW_HIGH();
#if LCD_TYP == 0
    actcntr == 1 ? ENA_HIGH() : ENA2_HIGH();
#else
    ENA_HIGH();
#endif
    delay_us(1);
    loop_until_bit_is_clear(BUSY_PIN_PORT,BUSY); // Wait until lcd is ready
#if LCD_TYP == 0
    actcntr == 1 ? ENA_LOW() : ENA2_LOW();
#else
    ENA_LOW();
#endif
    LCD_DDR_PORT |= (1<<BUSY);
}


//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

static void lcd_trig_enable( void )

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
{
    // Only if 4x40 lcd is selected
#if LCD_TYP == 0
    // Lcd controller 1
    if (actcntr == 1)
    {
        ENA_HIGH();
        delay_us(1);
        ENA_LOW();
    }

    // Lcd controller 2
    if (actcntr == 2)
    {
        ENA2_HIGH();
        delay_us(1);
        ENA2_LOW();
    }
#else
    ENA_HIGH();
    delay_us(1);
    ENA_LOW();
#endif
}



//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

extern void lcd_sendcom( unsigned char command, unsigned char cntr )

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
{
    lcd_waitbusy();
    RS_LOW();
    RW_LOW();

#if LCD_TYP == 0
    actcntr = cntr;
#endif

  // 8 bit mode part
#if LCD_MODE == 8
    LCD_DATA_PORT = command;
#endif

  // 4 bit mode part
#if LCD_MODE == 4
    // Send high byte
    LCD_DATA_PORT = (command & 0xF0) | (LCD_DATA_PORT & 0x0F);
    lcd_trig_enable();
    BUSY_PORT &= ~(1<<BUSY);
    // Send low byte
    LCD_DATA_PORT = ((command << 4) & 0xF0) | (LCD_DATA_PORT & 0x0F);
#endif
    lcd_trig_enable();
    LCD_DATA_PORT &= ~(1<<BUSY);
}



//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

extern void lcd_setcur( unsigned char ypos, unsigned char xpos )

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
{
#if LCD_TYP == 0
register unsigned char tmp;

    switch (ypos)
    {
        case 3:
                tmp = 2;
                ypos = 1;
                break;

        case 4:
                tmp = 2;
                ypos = 2;
                break;
        default:
                tmp = 1;
    }

    lcd_sendcom(pgm_read_byte(&screenpos[ypos-1]) + (xpos-1), tmp);
#else
    lcd_sendcom(pgm_read_byte(&screenpos[ypos-1]) + (xpos-1), 1);
#endif


}



//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

extern void lcd_putchar( unsigned char data )

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
{
    lcd_waitbusy();
    RS_HIGH();
    RW_LOW();

  // 8 bit mode part
#if LCD_MODE == 8
    LCD_DATA_PORT = data;
#endif

  // 4 bit mode part
#if LCD_MODE == 4
    // send high byte
    LCD_DATA_PORT = (data & 0xF0) | (LCD_DATA_PORT & 0x0F);
    lcd_trig_enable();
    // send low byte
    LCD_DATA_PORT = ((data << 4) & 0xF0) | (LCD_DATA_PORT & 0x0F);
#endif
    lcd_trig_enable();
    LCD_DATA_PORT &= ~(1<<BUSY);
}



//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

extern void lcd_putstr( unsigned char *data )

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
{
    register unsigned char c,i;

    i = MAXLINELENGHT;

    while ( (c = *data++) && i-- )
        lcd_putchar(c);
}



//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

extern void lcd_putstr_xy( unsigned char *data, unsigned char ypos, unsigned char xpos )

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
{
    lcd_setcur( ypos, xpos );
    lcd_putstr( data );
}


//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

extern void lcd_putstr_P( const char *data )

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
{
    register unsigned char c,i;

    i = MAXLINELENGHT;

    while ( (c = pgm_read_byte(data++)) && i--)
        lcd_putchar(c);
}



//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

extern void lcd_clearline( unsigned char line )

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
{
    register unsigned char i;

    lcd_setcur(line, 1);

    for (i=1; i <= MAXLINELENGHT; i++)
        lcd_putchar(0x20);

    lcd_setcur(line, 1);
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

extern void lcd_LoadCustomChar(u08* lcdCustomCharArray, u08 romCharNum, u08 lcdCharNum, u08 cntr)

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  
{
  register u08 i;

  // multiply the character index by 8
  lcdCharNum = (lcdCharNum<<3); // each character occupies 8 bytes
  romCharNum = (romCharNum<<3); // each character occupies 8 bytes

  // copy the 8 bytes into CG (character generator) RAM
  for(i=0; i<8; i++)
  {
    // set CG RAM address
    lcd_sendcom((LCD_CGRAM) | (lcdCharNum+i), cntr);
    // write character data
    lcd_putchar( pgm_read_byte(lcdCustomCharArray+romCharNum+i) );
  }

}



