/*
 * Copyright (c) 2008 Hans - Joachim Borchers
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE DEVELOPERS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE DEVELOPERS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <avr/pgmspace.h>
#include <stdio.h>

#include "typedef.h"
#include "sysdef.h"
#include "drvtimer.h"
#include "ioadc.h"
#include "drvtemp.h"
#include "cmdline.h"

#define TASK_CYCLE_TIME 200  // call the task every 200ms

static u16 uTempVal[E_SGUN_MAX];
static u16 uAdcVal[E_SGUN_MAX];
static u16 uTempMaxVal[E_SGUN_MAX];

static void drvTemp_Task( void )
{
  E_SOLDER_GUNS n;
  drvtim_TimerAdd( drvTemp_Task, TASK_CYCLE_TIME );
  for(n=E_SGUN1; n < E_SGUN_MAX; n++)
  {
    uTempVal[n] = drvtemp_ReadTempFromSensor(n, &uAdcVal[n] );
    if( uTempMaxVal[n] < uTempVal[n])
      uTempMaxVal[n] = uTempVal[n];  // store new maximum
  }
}

void drvtemp_InitModule( void )
{
  uTempMaxVal[0] = 0;
  uTempMaxVal[1] = 0;
  drvtim_TimerAdd( drvTemp_Task, TASK_CYCLE_TIME ); 
}

u16 drvtemp_ReadTempFromSensor( E_SOLDER_GUNS uSensor, u16 *uAdc )
{
  u16 tist;
  u32 adc;
  
  if( uSensor == E_SGUN1 )
    ioadc_StartConversion(0);
  else
    ioadc_StartConversion(1);
  
  adc = ioadc_ReadAdcValue();  // read temp
  *uAdc = adc;
  
  // temp = 4E-5 * x2 + 0.318 * x + 96.124
  if( uSensor == E_SGUN1 )
    tist = (4L * adc*adc + 31800L*adc + 9612400L) / 100000L;
  else
    tist = (4L * adc*adc + 31800L*adc + 9612400L) / 100000L;
  return tist;
}

u16 drvtemp_ReadTemp( E_SOLDER_GUNS uSensor, u16 *uAdc )
{
  *uAdc = uAdcVal[uSensor];
  return uTempVal[uSensor];
}

u16 drvtemp_ReadMaxTemp( E_SOLDER_GUNS uSensor )
{
  return uTempMaxVal[uSensor];
}
