/*
 * Copyright (c) 2008 Hans - Joachim Borchers
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE DEVELOPERS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE DEVELOPERS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <avr/io.h>

#include "typedef.h"
#include "sysdef.h"
#include "ioadc.h"

void ioadc_InitModule( void )
{

//--- Setup the ADC registers
  // ADMUX: REFS1, REFS0, ADLAR, MUX4, MUX3, MUX2, MUX1, MUX0
  ADMUX = 0b01000000;  // use AVCC as Vref 
  
  // ADCSRA:  ADEN, ADSC, ADATE, ADIF, ADIE, ADPS2, ADPS1, ADPS0
  ADCSRA = 0b10010111;   // enable ADC, enable ADC IRQ, prescaler=128
  
  ioadc_StartConversion(0);  // do one conversion for init the ADC
  
}

void ioadc_StartConversion( u08 uChannel )
{
  
  ADMUX = (ADMUX & 0xE0) | uChannel; 
  sbi(ADCSRA, ADSC);
  
  while( ADCSRA & BV(ADSC) );  // wait for finishing conversion
}

u16 ioadc_ReadAdcValue( void )
{
  u16 val;
  val = ADCL;
  val |= (ADCH << 8);
  return val;
}
