/*
 * Copyright (c) 2008 Hans - Joachim Borchers
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE DEVELOPERS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE DEVELOPERS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

// Timer module for ATMEGA16/644
#include <avr\interrupt.h>

#include "typedef.h"
#include "sysdef.h"
#include "drvkeys.h"

static void (*Fct1Ms)(void) = 0;

#ifdef __AVR_ATmega32__
s08 Iotimer_InitModule( void )
{
//--- Setup timer 0 for 1ms ticks
  // TCCR0: FOC0 WGM00 COM01 COM00 WGM01 CS02 CS01 CS00
  TCCR0 = 0b00001011;  //prescaler=64, CTC mode
  OCR0  = 250-1;  // output compare reg.
  
  
//--- Enable timer interrupts
  TIMSK = BV(OCIE0);
  
  return 0;
}
  

// Timer 0: Compare Match IRQ: 1ms tick
ISR( TIMER0_COMP_vect )
{
  if( Fct1Ms )
    Fct1Ms();
}
#endif

#ifdef __AVR_ATmega644__
s08 Iotimer_InitModule( void )
{
//--- Setup timer 0 for 1ms ticks
  // TCCR0A: 
  TCCR0A = (1<<WGM01);  //CTC mode
  TCCR0B = (1<<CS01)|(1<<CS00);  //prescaler=64
  
  OCR0A  = 25-1;  // output compare reg. A
  OCR0A  = 25-1;  // output compare reg. B
  
//--- Enable timer interrupts
  TIMSK0 = BV(OCIE0A);
  
  return 0;
}
  

// Timer 0: Compare Match IRQ: 1ms tick
ISR( TIMER0_COMPA_vect )
{
  static u08 div = 10;
  drvkeys_task();
  if( !(--div) )
  {
    div = 10;
    if( Fct1Ms )
      Fct1Ms();
  }
}
#endif


void Iotimer_RegisterTimer0Fct( void (*fct)(void) )
{
  Fct1Ms = fct;
}
