/*
 * Copyright (c) 2008 Hans - Joachim Borchers
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE DEVELOPERS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE DEVELOPERS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <avr/pgmspace.h>
#include <string.h>
#include <stdio.h>

#include "typedef.h"
#include "sysdef.h"
#include "drvheat.h"
#include "drvtemp.h"
#include "drveeprom.h"
#include "cmdline.h"
#include "pid.h"
#include "xitoa.h"

u08 reg_dump_temp[2]; // dump temperatur on RS232
u08 heat_pwr_abs[2];  // if |= 0, this is the fixed heating power in %, which is set to
                      // the PWM!!!
static u08 uPwrOnHeatUp = TRUE;

static void heat_control_loop( void );

void reg_InitModule( void )
{
  ;
}


void reg_Task( void )
{
  if( drvheat_HeatCycleEnd() )
    heat_control_loop();
}


static void heat_control_loop( void )
{
  static u16 loops=0;
  E_SOLDER_GUNS n;
  u16 tist=0,tsoll=0,adc=0;
  u08 stell=0;
  
  for(n=E_SGUN1; n<E_SGUN_MAX; n++)
  {
    
    if( !drveeprom_GetParameter()->tEeprom->uGunOff[n] )
    {
      //--- read temperature  
      tist = drvtemp_ReadTemp( n, &adc );
      //--- read nominal value
      tsoll = drveeprom_GetParameter()->tEeprom->uNominalTemp[n];
    
      //--- call the PID  
      stell = PID_Cyclic (n, tist, tsoll);
  
      if( uPwrOnHeatUp && n==E_SGUN1 )
      {
        if( (tsoll - tist) < 30 )
          uPwrOnHeatUp = FALSE;
      }
  
      // In case of power on phase set power of heater 2 to 0!
      if( uPwrOnHeatUp && n==E_SGUN2 )
        stell = 0;  
      
      if( heat_pwr_abs[n] != 0 )
        stell = heat_pwr_abs[n];
        
      //--- set heating power  
      drvheat_SetHeatingPower( n, stell );
    }
    else
    {
      if( n== E_SGUN1 )
        uPwrOnHeatUp = FALSE;
      
      drvheat_SetHeatingPower( n, 0 );
    }
    
    if( reg_dump_temp [n] && !cmdlineEditActive() )
      xprintf( PSTR("%d;%4d;%3d;%3d;%3d;%3d\r\n"),n,loops,tsoll,tist,adc,stell);
  
  }
  loops++;
}
